////////////////////////////////////////////////////////////////////////////
//
//  FPGA_DAP.PAS
//
//  Diese Klasse stellt alle grundlegenden Funktionen des DAPs zur Verfgung
//  Dazu zhlen Muten/Entmuten des Ausgangs (IO),
//  Beschreiben aller Speicherbereiche (PMEM, CMEM, DMEM, WMEM, MMEM, UMEM) tlw.
//  mit aussage-krftigen funktionen.
//  Erstellen von Programmen mit PMEM_xxx Funktionen
//  Schreiben von Koeffizienten mit Hidden Write
//  Beschreiben des Datenspeichers mit entsprechenden Funktionen (Oszillator, Shaper, Delay)
//
//  In Zukunft werden noch Funktionen fr die Manipulation des MicroCode-Speichers
//  mit aussagekrftigen Funktionen bereitgestellt.
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit fpga_dap;


interface


uses windows, classes, sysutils, fpga_com_base, codec_interface;

type
     // Datenstruktur um einen Biquad (5 32 Bit-Werte) zu schreiben

     TDAPBiquad = record
       ulB : array [0 .. 2] of longint;
       ulA : array [0 .. 1] of longint;
     end;

     // DAP-klasse
     TFPGADAP = class (TFPGADevice)
     public
       constructor Create  (base : TFPGACOMBase);
       destructor Destroy; override; 

       // DAP Muten/Entmuten
       procedure Mute (mute : boolean);

       // Speicher allgemein beschreiben
       procedure WriteMEMStart  (adr : integer);
       procedure WriteMEMLong   (val : dword);
       procedure WriteMEMWord   (val : word);
       procedure WriteMEMEnd;

       // CMEM-Speicher Operationen (LONG Wert, Biquad und LONG-Buffer schreiben)

       procedure CMEM_Longint (Adr : byte; Coeff : longint);
       procedure CMEM_Biquad (Adr : byte; Coeff : TDAPBiquad);
       procedure CMEM_Buffer (Adr : byte; ptr : pointer; len : integer);


       // Funktionen zum erstellen von Audio-Programmen

       // Programmspeicher lschen
       procedure ClearPMEM;
       // einzelnen Opcode Schreiben
       procedure WritePMEMOpcode (opc, core, eop, inp, outp, amem, cmem : byte);

       // Programm-Speicher Funktionen (Opcode-Ebene)
       procedure PMEM_StartProgram;    // Programm-Start
       procedure PMEM_Nop    (inp, outp             : byte);
       procedure PMEM_Biquad (inp, outp, amem, cmem : byte);
       procedure PMEM_Store  (inp, outp, amem       : byte);
       procedure PMEM_Load   (     outp,       cmem : byte);
       procedure PMEM_Move   (     outp, amem, cmem : byte);
       procedure PMEM_ClrSum (     outp             : byte);
       procedure PMEM_AddInp (inp, outp             : byte);
       procedure PMEM_AddMem (     outp,       cmem : byte);
       procedure PMEM_Volume (inp, outp,       cmem : byte);
       procedure PMEM_MixInp (inp, outp,       cmem : byte);
       procedure PMEM_MixMem (     outp, amem, cmem : byte);
       procedure PMEM_Osc    (     outp, amem       : byte);
       procedure PMEM_Shaper (inp, outp, amem       : byte);
       procedure PMEM_Delay  (inp, outp, amem       : byte);
       procedure PMEM_EndProgram;    // Programm Ende


       // Schreiben von 32 Bit Werten in den DMEM Speicherbereich
       procedure DMEM_Write (adr : byte; val : integer);

       // DMEM Funktionen (fr DDS-Oszillator, Wave-Shaper und Delay)
       procedure DMEM_Osc   (adr : byte; val : dword);
       procedure DMEM_Shape (adr : byte; wave : byte; pwm : byte);
       procedure DMEM_Delay (adr : byte; base, offs, mask : integer);


     end;

// Konstanten

// Speicher-Zugriffe

const DAP_WRITE_MEM : byte = $0;  // Speicher schreiben
const DAP_READ_MEM  : byte = $1;  //   "  "   lesen (noch nicht implementiert)
const DAP_WRITE_IO  : byte = $2;  // Memory Mapped IO schreiben
const DAP_READ_IO   : byte = $3;  //   ""     ""    " lesen (noch nicht implementiert)

// Eingnge fr Opcodes

const DAP_IN_1      : byte = $0;  // Input  1 = Codec 1 Kanal L
const DAP_IN_2      : byte = $1;  // Input  2 = Codec 1 Kanal R
const DAP_IN_3      : byte = $2;  // Input  3 = Codec 2 Kanal L
const DAP_IN_4      : byte = $3;  // Input  4 = Codec 2 Kanal R
const DAP_IN_UC     : byte = $e;  // Input 14 = Ausgang des UTIL-Cores
const DAP_IN_SC     : byte = $f;  // Input 14 = Ausgang des SDSP-Cores

// Ausgnge fr Opcodes

const DAP_OUT_SC_1  : byte = $0;  // SDSP-Core -> Kanal 1 = Codec 1 Kanal L
const DAP_OUT_SC_2  : byte = $1;  // SDSP-Core -> Kanal 2 = Codec 1 Kanal R
const DAP_OUT_SC_3  : byte = $2;  // SDSP-Core -> Kanal 3 = Codec 2 Kanal L
const DAP_OUT_SC_4  : byte = $3;  // SDSP-Core -> Kanal 4 = Codec 2 Kanal R

const DAP_OUT_UC_1  : byte = $4;  // UTIL-Core -> Kanal 1 = Codec 1 Kanal L
const DAP_OUT_UC_2  : byte = $5;  // UTIL-Core -> Kanal 2 = Codec 1 Kanal R
const DAP_OUT_UC_3  : byte = $6;  // UTIL-Core -> Kanal 3 = Codec 2 Kanal L
const DAP_OUT_UC_4  : byte = $7;  // UTIL-Core -> Kanal 4 = Codec 2 Kanal R

// Opcodes

const DAP_OPC_NOP     : byte = $0; // NOP
const DAP_OPC_BIQUAD  : byte = $1; // Biquad durchrechnen
const DAP_OPC_STORE   : byte = $2; // Audio-Wert an AMEM Adresse speichern
const DAP_OPC_LOAD    : byte = $3; // Audio-Wert von AMEM Adresse laden
const DAP_OPC_MOVE    : byte = $4; // Audio-Wert kopieren (von CMEM-Adresse nach AMEM-Adresse)
const DAP_OPC_CLRSUM  : byte = $5; // Summen Signal (Akkumulator) lschen
const DAP_OPC_ADDINP  : byte = $6; // Eingang zu Summe addieren
const DAP_OPC_MIXINP  : byte = $7; // Eingang zu Summe mischen (mit Lautstrke Koeffizient)
const DAP_OPC_VOLUME  : byte = $8; // Lautstrke
const DAP_OPC_ADDMEM  : byte = $9; // AMEM Wert zu Summe addieren
const DAP_OPC_MIXMEM  : byte = $a; // AMEM Wert zu Summe mischen (mit Lautstrke Koeffizient)

const DAP_OPC_OSC     : byte = $1; // DDS rechnen
const DAP_OPC_SHAPER  : byte = $2; // Shaper fr DDS
const DAP_OPC_DELAY   : byte = $3; // Delay ausfhren


// Konstanten fr Speicherbereiche

// Rahmen
const DAP_MEM_PMEM    : word = $0000;  // PMEM Programm-Speicher

// SDSP-Core
const DAP_MEM_CMEM    : word = $2000;  // CMEM Koeffizienten Speicher (write only und hidden write beachten)
const DAP_MEM_MMEM    : word = $4000;  // MMEM Microcode Speicher fr SDSP-Core

// UTIL-Core
const DAP_MEM_DMEM    : word = $6000;  // DMEM Daten Speicher fr Util Core
const DAP_MEM_WMEM    : word = $8000;  // WMWM Wavetable Memory fr Shaper
const DAP_MEM_UMEM    : word = $a000;  // UMEM Microcode Speicher fr Util-Core


implementation





constructor TFPGADAP.Create (base : TFPGACOMBase);
begin
  inherited Create (base);
end;

destructor TFPGADAP.Destroy;
begin
  inherited;
end;


procedure TFPGADAP.WriteMEMStart (adr : integer);
begin
  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + chr (hi (adr)) + chr (lo (adr)));
end;

procedure TFPGADAP.WriteMEMLong (val : dword);
var x : ansistring;
begin
  x :=  chr ( (val shr  0) and $ff) + chr ( (val shr  8) and $ff) +
        chr ( (val shr 16) and $ff) + chr ( (val shr 24) and $ff);
  SendData (x);
end;


procedure TFPGADAP.WriteMEMWord (val : word);
var x : ansistring;
begin
  x :=  chr ( (val shr  0) and $ff) + chr ( (val shr  8) and $ff);
  SendData (x);
end;

procedure TFPGADAP.WriteMEMEnd;
begin
  DeSelectFPGA;
end;




procedure TFPGADAP.Mute (mute : boolean);
var x : byte;
begin
  if mute then x := 1 else x := 0;
  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_IO) + chr ($00) + chr (x));
  DeSelectFPGA;
end;

procedure TFPGADAP.CMEM_Longint (Adr : byte; Coeff : longint);
var x : ansistring;
begin
  x := '';
  AppendULONG(x, coeff);

  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$00 + x);
  DeSelectFPGA;
  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$20 + chr (adr) + chr (1) + #$00#$00);
  DeSelectFPGA;
end;


procedure TFPGADAP.CMEM_Biquad (Adr : byte; Coeff : TDAPBiquad);
var x : ansistring;
begin
  x := '';
  AppendULONG(x, coeff.ulB [0]);
  AppendULONG(x, coeff.ulB [1]);
  AppendULONG(x, coeff.ulB [2]);
  AppendULONG(x, coeff.ulA [0]);
  AppendULONG(x, coeff.ulA [1]);

  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$00 + x);
  DeSelectFPGA;
  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$20 + chr (adr) + chr (5) + #$00#$00);
  DeSelectFPGA;
end;


procedure TFPGADAP.CMEM_Buffer (Adr : byte; ptr : pointer; len : integer);
var x : ansistring;
    c : integer;
begin
  x := '';

  for c := 0 to len - 1 do
  begin
    AppendULONG (x, longint (ptr^));
    ptr := pointer (longint (ptr) + sizeof (longint));
  end;

  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$00 + x);
  DeSelectFPGA;
  SelectFPGA;
  SendData    (deviceport + chr (DAP_WRITE_MEM) + #$20#$20 + chr (adr) + chr (len) + #$00#$00);
  DeSelectFPGA;
end;






procedure TFPGADAP.WritePMEMOpcode (opc, core, eop, inp, outp, amem, cmem : byte);
begin
  opc := opc and $f; core := core and $1; eop := eop and $1;
  inp := inp and $f; outp := outp and $f;
  WriteMEMLong ((opc shl 28) or (core shl 25) or (eop  shl 24) or
                (inp shl 20) or (outp shl 16) or (amem shl  8) or (cmem shl  0));
end;

procedure TFPGADAP.ClearPMEM;
var adr : integer;
begin
  WriteMEMStart($0000);
  for adr := 0 to 511 do
  begin
    WriteMEMLong($01000000);
  end;
  WriteMEMEnd;
end;


procedure TFPGADAP.PMEM_StartProgram;
begin
  WriteMEMStart($0000);
end;


procedure TFPGADAP.PMEM_Nop (inp, outp : byte);
begin
  WritePMEMOpcode (DAP_OPC_NOP, 0, 0, inp, outp, 0, 0);
end;

procedure TFPGADAP.PMEM_Biquad (inp, outp, amem, cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_BIQUAD, 0, 0, inp, outp, amem, cmem);
end;

procedure TFPGADAP.PMEM_Store  (inp, outp, amem       : byte);
begin
  WritePMEMOpcode (DAP_OPC_STORE, 0, 0, inp, outp, amem, 0);
end;

procedure TFPGADAP.PMEM_Load   (     outp,       cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_LOAD, 0, 0, 0, outp, 0, cmem);
end;

procedure TFPGADAP.PMEM_Move   (     outp, amem, cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_MOVE, 0, 0, 0, outp, amem, cmem);
end;

procedure TFPGADAP.PMEM_ClrSum (     outp             : byte);
begin
  WritePMEMOpcode (DAP_OPC_CLRSUM, 0, 0, 0, outp, 0, 0);
end;

procedure TFPGADAP.PMEM_AddInp (inp, outp             : byte);
begin
  WritePMEMOpcode (DAP_OPC_ADDINP, 0, 0, inp, outp, 0, 0);
end;

procedure TFPGADAP.PMEM_AddMem (     outp,       cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_ADDMEM, 0, 0, 0, outp, 0, cmem);
end;

procedure TFPGADAP.PMEM_Volume (inp, outp,       cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_VOLUME, 0, 0, inp, outp, 0, cmem);
end;

procedure TFPGADAP.PMEM_MixInp (inp, outp,       cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_MIXINP, 0, 0, inp, outp, 0, cmem);
end;

procedure TFPGADAP.PMEM_MixMem (     outp, amem, cmem : byte);
begin
  WritePMEMOpcode (DAP_OPC_MIXMEM, 0, 0, 0, outp, amem, cmem);
end;

procedure TFPGADAP.PMEM_Osc    (     outp, amem       : byte);
begin
  WritePMEMOpcode (DAP_OPC_OSC, 1, 0, 0, outp, amem, 0);
end;

procedure TFPGADAP.PMEM_Shaper (inp, outp, amem       : byte);
begin
  WritePMEMOpcode (DAP_OPC_SHAPER, 1, 0, inp, outp, amem, 0);
end;

procedure TFPGADAP.PMEM_Delay  (inp, outp, amem       : byte);
begin
  WritePMEMOpcode (DAP_OPC_DELAY, 1, 0, inp, outp, amem, 0);
end;

procedure TFPGADAP.PMEM_EndProgram;
begin
  WritePMEMOpcode (DAP_OPC_NOP, 0, 1, 0, 0, 0, 0);
  WriteMEMEnd;
end;




procedure TFPGADAP.DMEM_Write (adr : byte; val : integer);
begin
  WriteMEMStart(DAP_MEM_DMEM + (adr shl 2));
  WriteMEMLong (val);
  WriteMEMEnd;
end;

procedure TFPGADAP.DMEM_Osc   (adr : byte; val : dword);
begin
  DMEM_Write(adr + 1, val);
end;

procedure TFPGADAP.DMEM_Shape (adr : byte; wave : byte; pwm : byte);
begin
  DMEM_Write (adr, (wave and $f) or (pwm shl 4) );
end;

procedure TFPGADAP.DMEM_Delay (adr : byte; base, offs, mask : integer);
begin
  DMEM_Write (adr    , base);
  DMEM_Write (adr + 1, 0   );
  DMEM_Write (adr + 2, offs);
  DMEM_Write (adr + 3, mask);
end;




end.
